/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * \package    RobotAPI::gui-plugins::DebugDrawerViewerWidgetController
 * \author     Mirko Waechter ( mirko dot waechter at kit dot edu )
 * \date       2016
 * \copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "DebugDrawerViewerWidgetController.h"
#include <ArmarXCore/core/ArmarXManager.h>
#include <string>

using namespace armarx;

DebugDrawerViewerWidgetController::DebugDrawerViewerWidgetController()
{
    rootVisu = NULL;
    widget.setupUi(getWidget());

}


DebugDrawerViewerWidgetController::~DebugDrawerViewerWidgetController()
{

}


void DebugDrawerViewerWidgetController::loadSettings(QSettings* settings)
{

}

void DebugDrawerViewerWidgetController::saveSettings(QSettings* settings)
{

}


void DebugDrawerViewerWidgetController::onInitComponent()
{
    rootVisu = new SoSeparator;
    rootVisu->ref();


    enableMainWidgetAsync(false);
    connect(widget.btnClearAll, SIGNAL(clicked()), this, SLOT(on_btnClearAll_clicked()), Qt::UniqueConnection);
}


void DebugDrawerViewerWidgetController::onConnectComponent()
{
    // create the debugdrawer component
    std::string debugDrawerComponentName = "GUIDebugDrawer_" + getName();
    ARMARX_INFO << "Creating component " << debugDrawerComponentName;
    debugDrawer = Component::create<DebugDrawerComponent>(properties, debugDrawerComponentName);

    if (mutex3D)
    {
        //ARMARX_IMPORTANT << "mutex3d:" << mutex3D.get();
        debugDrawer->setMutex(mutex3D);
    }
    else
    {
        ARMARX_ERROR << " No 3d mutex available...";
    }

    ArmarXManagerPtr m = getArmarXManager();
    m->addObject(debugDrawer, false);

    {
        boost::recursive_mutex::scoped_lock lock(*mutex3D);
        rootVisu->addChild(debugDrawer->getVisualization());
    }
    enableMainWidgetAsync(true);
}


SoNode *DebugDrawerViewerWidgetController::getScene()
{
    return rootVisu;
}


void armarx::DebugDrawerViewerWidgetController::onExitComponent()
{
    if (rootVisu)
    {
        rootVisu->removeAllChildren();
        rootVisu->unref();
        rootVisu = NULL;
    }
}

void armarx::DebugDrawerViewerWidgetController::on_btnClearAll_clicked()
{
    ARMARX_INFO << "Clearing visu";
    if(debugDrawer)
    {
        debugDrawer->clearAll();
    }
}
